<?php

if ( !defined('ABSPATH') ) exit;

function quickcab_add_fixed_address_meta_box() {
  add_meta_box('quickcab_fixed_address_options', esc_html__('Fixed Address Options', 'quickcab'),  'quickcab_render_fixed_address_options_meta_box', array('fixed_address'), 'normal', 'default');
}
add_action('add_meta_boxes', 'quickcab_add_fixed_address_meta_box');

function quickcab_render_fixed_address_options_meta_box() {
  global $post;

  do_action('quickcab_meta_box_fixed_address_options_fields', $post->ID);

  wp_nonce_field(basename(__FILE__), 'quickcab_fixed_address_meta_box_nonce');
}

function quickcab_render_fixed_address_options_fields($post_id) {
  $screen = get_current_screen();
  $is_new = $screen->action == 'add';

  if ( !$is_new ) {
    $fixed_address = new QuickCab_Fixed_Address($post_id);
  }

  ?><table class="form-table">
    <tbody><?php
      quickcab_render_fixed_address_metabox_string_row(
        'quickcab_fixed_address_name',
        $post_id,
        array(
          'text' => esc_html__('Name of Destination', 'quickcab')
        ),
        array(
          'required' => true
        ),
        array(
          'text' => esc_html__('The name of the destination that customers will see on the booking form.', 'quickcab')
        )
      );
      quickcab_render_fixed_address_metabox_autocomplete_row(
        'quickcab_fixed_address_address',
        $post_id,
        array(
          'text' => esc_html__('Address of Destination', 'quickcab')
        ),
        array(
          'required' => true
        ),
        array(
          'text' => esc_html__('The address of the destination. Ensure the tick is green before saving.', 'quickcab')
        )
      );
    ?></tbody>
    <tbody>
      <tr>
        <th scope="row">
          <label><?php
            echo esc_html__('Starting or Destination Address', 'quickcab');
          ?></label>
        </th>
        <td>
          <fieldset>
            <legend class="screen-reader-text">
              <span><?php echo esc_html__('Origin Address', 'quickcab'); ?></span>
            </legend>
            <label for="quickcab_fixed_address_is_origin">
              <input name="quickcab_fixed_address_is_origin" type="checkbox" id="quickcab_fixed_address_is_origin" <?php if (!$is_new && $fixed_address->is_origin()) { ?>checked<?php } ?>>
              <?php echo esc_html__('Starting Address', 'quickcab'); ?>
            </label>
          </fieldset>

          <fieldset>
            <legend class="screen-reader-text">
              <span><?php echo esc_html__('Destination Address', 'quickcab'); ?></span>
            </legend>
            <label for="quickcab_fixed_address_is_destination">
              <input name="quickcab_fixed_address_is_destination" type="checkbox" id="quickcab_fixed_address_is_destination" <?php if (!$is_new && $fixed_address->is_destination()) { ?>checked<?php } ?>>
              <?php echo esc_html__('Destination Address', 'quickcab'); ?>
            </label>
          </fieldset>
          <p class="description"><?php
            echo esc_html__('Which field do you want this address to be available for?', 'quickcab');
          ?></p>
        </td>
      </tr>
    </tbody>
  </table><?php
}
add_action('quickcab_meta_box_fixed_address_options_fields', 'quickcab_render_fixed_address_options_fields');

function quickcab_fixed_address_metabox_fields() {
  return apply_filters('quickcab_metabox_fixed_address_fields_save', array(
    'quickcab_fixed_address_name',
    'quickcab_fixed_address_address',
    'quickcab_fixed_address_is_origin',
    'quickcab_fixed_address_is_destination'
  ));
}

function quickcab_fixed_address_meta_box_save($post_id, $post) {
  if ( !isset($_POST['quickcab_fixed_address_meta_box_nonce']) || !wp_verify_nonce($_POST['quickcab_fixed_address_meta_box_nonce'], basename(__FILE__)) ) {
    return;
  }

  if ( (defined('DOING_AUTOSAVE') && DOING_AUTOSAVE) || (defined('DOING_AJAX') && DOING_AJAX) || isset($_REQUEST['bulk_edit']) ) {
    return;
  }

  if ( isset($post->post_type) && 'revision' == $post->post_type ) {
    return;
  }

  $fixed_address = new QuickCab_Fixed_Address($post_id);

  // The default fields that get saved
  foreach ( quickcab_fixed_address_metabox_fields() as $field ) {
    $meta_name = str_replace('quickcab_fixed_address_', '', $field);

    if ( empty($_POST[$field]) ) {
      $fixed_address->delete_meta($meta_name);
    } else {
      $new = apply_filters('quickcab_fixed_address_metabox_save_' . $meta_name, $_POST[$field], $post_id);
      $fixed_address->update_meta($meta_name, $new);
    }
  }

  do_action('quickcab_save_fixed_address', $post_id, $post);
}
add_action('save_post', 'quickcab_fixed_address_meta_box_save', 10, 2);

/**
 * Update the post title to the name of the address
 *
 * @since  1.0
 * @param  int     $post_id  ID of the post
 * @param  WP_Post $post     the WP_Post
 * @return void
 */
function quickcab_fixed_address_update_title($post_id, $post) {
  global $wpdb;
  $fixed_address = new QuickCab_Fixed_Address($post_id);
  $title = $fixed_address->get_name();

  $wpdb->update(
    $wpdb->posts,
    array('post_title' => $title),
    array('ID' => $post_id)
  );
}
add_action('quickcab_save_fixed_address', 'quickcab_fixed_address_update_title', 10, 2);

function quickcab_fixed_address_change_is_origin($value, $post_id) {
  return $value == 'on';
}
add_filter('quickcab_fixed_address_metabox_save_is_origin', 'quickcab_fixed_address_change_is_origin', 10, 2);

function quickcab_fixed_address_change_is_destination($value, $post_id) {
  return $value == 'on';
}
add_filter('quickcab_fixed_address_metabox_save_is_destination', 'quickcab_fixed_address_change_is_destination', 10, 2);
