<?php

if ( !defined('ABSPATH') ) exit;

function quickcab_geofence_rule_columns($original_columns) {
  $new_columns = array(
    'cb'        => $original_columns['cb'],
    'title'     => esc_html__('Name', 'quickcab'),
    'price'     => esc_html__('Price', 'quickcab'),
    'vehicles'  => esc_html__('Vehicles', 'quickcab')
  );

  if ( quickcab_get_option('enable_static_map_geofence_preview') == 'on' ) {
    $new_columns['locations'] = esc_html__('Locations', 'quickcab');
  }

  return apply_filters('quickcab_geofence_rule_columns', $new_columns);
}
add_filter('manage_geofence_rule_posts_columns', 'quickcab_geofence_rule_columns');

function quickcab_render_geofence_rule_columns($column_name, $post_id) {
  if ( get_post_type($post_id) == 'geofence_rule' ) {
    $geofence_rule = new QuickCab_Geofence_Rule($post_id);

    switch ( $column_name ) {
      case 'price':
        switch ( $geofence_rule->get_pricing_structure() ) {
          case 'fixed-price':
            echo quickcab_format_price(esc_html($geofence_rule->get_fixed_price_amount()));
            break;
          case 'increased-price':
            echo '+' . quickcab_format_price(esc_html($geofence_rule->get_increase_price_amount()));
            break;
        }
        break;
      case 'locations':
        ?><div class="quickcab-geofence-index__map-container"><?php
          if ( $geofence_rule->starting_area_map() ) {
            ?><img src="<?php echo esc_attr($geofence_rule->get_starting_map_image_path()); ?>" class="quickcab-geofence-index__preview-map"><?php
          } else {
            ?><div class="quickcab-geofence-index__anywhere">
              <div class="quickcab-text"><?php echo esc_html__('Anywhere', 'quickcab'); ?></div>
            </div><?php
          }

          if ( $geofence_rule->destination_area_map() ) {
            ?><img src="<?php echo esc_attr($geofence_rule->get_destination_map_image_path()); ?>" class="quickcab-geofence-index__preview-map"><?php
          } else {
            ?><div class="quickcab-geofence-index__anywhere">
              <div class="quickcab-text"><?php echo esc_html__('Anywhere', 'quickcab'); ?></div>
            </div><?php
          }
        ?></div><?php
        break;
      case 'vehicles':
        $vehicle_ids = $geofence_rule->get_vehicle_ids();
        $vehicles_to_show = 15;

        if ( in_array(0, $vehicle_ids) ) {
          echo esc_html__('All Vehicles', 'quickcab');
        } else {
          $count = 0;

          foreach ( array_slice($vehicle_ids, 0, $vehicles_to_show) as $vehicle_id ) {
            $vehicle = new QuickCab_Vehicle($vehicle_id);

            ?><a href="<?php echo esc_attr(get_edit_post_link($vehicle_id)); ?>"><?php
              echo esc_html($vehicle->get_name());
            ?></a><?php

            if ( $count < min(count($vehicle_ids) - 1, $vehicles_to_show - 1) ) {
              echo ', ';
            }

            $count++;
          }

          if ( count($vehicle_ids) > $vehicles_to_show ) {
            echo ' ' . esc_html__('and more', 'quickcab') . '...';
          }
        }

        break;
    }
  }
}
add_action('manage_posts_custom_column', 'quickcab_render_geofence_rule_columns', 10, 2);
