<?php

if ( !defined('ABSPATH') ) exit;

/**
 * Display settings page
 */
function quickcab_render_settings_page() {
  $active_tab = isset($_GET['tab']) ? sanitize_text_field($_GET['tab']) : 'general';
  $active_section = 'main';

  $tabs = quickcab_get_registered_settings();
  $sections = $tabs[$active_tab]['sections'];

  if (
    isset($_GET['tab']) &&
    isset($_GET['section']) &&
    !empty($sections) &&
    array_key_exists($_GET['section'], $sections)
  ) {
    $active_section = sanitize_text_field($_GET['section']);
  }

  ?><div class="wrap">

    <h2 class="nav-tab-wrapper"><?php
      foreach ( $tabs as $tab_id => $settings_page ) {
        $tab_url = add_query_arg(array(
          'tab' => $tab_id
        ));
        $tab_url = remove_query_arg('section', $tab_url);

        $active = $active_tab == $tab_id ? 'nav-tab-active' : '';

        ?><a href="<?php echo esc_url($tab_url); ?>" class="nav-tab <?php echo esc_attr($active); ?>"><?php
          echo esc_html($settings_page['title']);
        ?></a><?php
      }
    ?></h2><?php

    if ( count($sections) > 1 ) {
      ?><div>
        <ul class="subsubsub"><?php
          $counter = 0;
          foreach ( $sections as $section_id => $section ) {
            $counter++;
            $tab_url = add_query_arg(array(
              'tab' => $active_tab,
              'section' => $section_id
            ));

            $class = $section_id == $active_section ? 'current' : '';

            ?><li>
              <a href="<?php echo esc_url($tab_url); ?>" class="<?php echo esc_attr($class); ?>"><?php
                echo esc_html($section['title']);
              ?></a><?php

              if ( $counter < count($sections) ) {
                ?> | <?php
              }
            ?></li><?php
          }
        ?></ul>
      </div><?php
    }

    ?><div id="tab_container">

      <form method="post" action="options.php">
        <table class="form-table"><?php
          settings_fields('quickcab_settings');

          do_settings_sections('quickcab_settings_' . $active_tab . '_' . $active_section);
        ?></table><?php

        submit_button();
      ?></form>

    </div>
  </div><?php
}
