<?php

if ( !defined('ABSPATH') ) exit;

class QuickCab_Booking_Form {

  public $ID = 0;
  public $post_author = 0;
  public $post_date = '0000-00-00 00:00:00';
  public $post_date_gmt = '0000-00-00 00:00:00';
  public $post_content = '';
  public $post_title = '';
  public $post_excerpt = '';
  public $post_status = 'publish';
  public $comment_status = 'open';
  public $ping_status = 'open';
  public $post_password = '';
  public $post_name = '';
  public $to_ping = '';
  public $pinged = '';
  public $post_modified = '0000-00-00 00:00:00';
  public $post_modified_gmt = '0000-00-00 00:00:00';
  public $post_content_filtered = '';
  public $post_parent = 0;
  public $guid = '';
  public $menu_order = 0;
  public $post_mime_type = '';
  public $comment_count = 0;
  public $filter;

  private $primary_color;
  private $secondary_color;
  private $map_starting_location;
  private $booking_time_interval;
  private $fixed_address_collection;
  private $fixed_address_destination;
  private $enforce_autocomplete_restriction;
  private $autocomplete_country;
  private $enable_extra_waiting_time;
  private $extra_waiting_time_unit;
  private $extra_waiting_time_max_value;
  private $enable_waypoints;
  private $enable_zoom_control;
  private $enable_map_type_control;
  private $enable_street_view_control;
  private $enable_full_screen_control;
  private $enable_pan_control;

  private $wp_post;
  private $meta_prefix;

  public function __construct($id) {
    $this->meta_prefix = '_quickcab_booking_form_';
    $booking_form = WP_Post::get_instance($id);

    if ( $booking_form ) {
      $this->setup_booking_form($booking_form);
    } else {
      return false;
    }
  }

  /**
   * Return the ID of the WP_Post object
   *
   * @since  1.0
   * @return int  order ID
   */
  public function get_ID() {
    return $this->ID;
  }

  public function get_primary_color() {
    return apply_filters('quickcab_booking_form_get_primary_color', $this->primary_color, $this->get_ID());
  }

  public function get_secondary_color() {
    return apply_filters('quickcab_booking_form_get_secondary_color', $this->secondary_color, $this->get_ID());
  }

  public function get_map_starting_location() {
    return apply_filters('quickcab_booking_form_get_map_starting_location', $this->map_starting_location, $this->get_ID());
  }

  public function get_booking_time_interval() {
    return apply_filters('quickcab_booking_form_get_booking_time_interval', $this->booking_time_interval, $this->get_ID());
  }

  public function get_fixed_address_collection() {
    return apply_filters('quickcab_booking_form_get_fixed_address_collection', $this->fixed_address_collection, $this->get_ID());
  }

  public function get_fixed_address_destination() {
    return apply_filters('quickcab_booking_form_get_fixed_address_destination', $this->fixed_address_destination, $this->get_ID());
  }

  public function get_enforce_autocomplete_restriction() {
    return apply_filters('quickcab_booking_form_get_enforce_autocomplete_restriction', $this->enforce_autocomplete_restriction, $this->get_ID());
  }
  
  public function get_autocomplete_country() {
    return apply_filters('quickcab_booking_form_get_autocomplete_country', $this->autocomplete_country, $this->get_ID());
  }

  public function enable_extra_waiting_time() {
    return apply_filters('quickcab_booking_form_enable_extra_waiting_time', $this->enable_extra_waiting_time, $this->get_ID());
  }

  public function get_extra_waiting_time_unit() {
    return apply_filters('quickcab_booking_form_extra_waiting_time_unit', $this->extra_waiting_time_unit, $this->get_ID());
  }

  public function get_extra_waiting_time_unit_i18n() {
    if ( $this->get_extra_waiting_time_unit() == 'minutes' ) {
      $translated_string = esc_html__('minutes', 'quickcab');
    } else {
      $translated_string = esc_html__('hours', 'quickcab');
    }

    return apply_filters('quickcab_booking_form_extra_waiting_time_unit_i18n', $translated_string, $this->get_ID());
  }

  public function get_extra_waiting_time_max_value() {
    return apply_filters('quickcab_booking_form_extra_waiting_time_max_value', $this->extra_waiting_time_max_value, $this->get_ID());
  }

  public function enable_waypoints() {
    return apply_filters('quickcab_booking_form_enable_waypoints', $this->enable_waypoints, $this->get_ID());
  }

  public function enable_zoom_control() {
    return apply_filters('quickcab_booking_form_enable_zoom_control', $this->enable_zoom_control, $this->get_ID());
  }

  public function enable_map_type_control() {
    return apply_filters('quickcab_booking_form_enable_map_type_control', $this->enable_map_type_control, $this->get_ID());
  }

  public function enable_street_view_control() {
    return apply_filters('quickcab_booking_form_enable_street_view_control', $this->enable_street_view_control, $this->get_ID());
  }

  public function enable_full_screen_control() {
    return apply_filters('quickcab_booking_form_enable_full_screen_control', $this->enable_full_screen_control, $this->get_ID());
  }

  public function enable_pan_control() {
    return apply_filters('quickcab_booking_form_enable_pan_control', $this->enable_pan_control, $this->get_ID());
  }

  public function show_distance() {
    return apply_filters('quickcab_booking_form_show_distance', $this->show_distance, $this->get_ID());
  }

  public function show_duration() {
    return apply_filters('quickcab_booking_form_show_duration', $this->show_duration, $this->get_ID());
  }

  public function enable_passengers_field() {
    return apply_filters('quickcab_booking_form_enable_passengers_field', $this->enable_passengers_field, $this->get_ID());
  }

  public function enable_suitcases_field() {
    return apply_filters('quickcab_booking_form_enable_suitcases_field', $this->enable_suitcases_field, $this->get_ID());
  }

  public function enable_direction_field() {
    return apply_filters('quickcab_booking_form_enable_direction_field', $this->enable_direction_field, $this->get_ID());
  }

  /**
   * Gets all QuickCab_Booking_Forms
   *
   * @since  1.0
   * @return array  array of all booking form
   */
  public static function find_all() {
    $booking_forms = array();

    $post_search = new WP_Query(array(
      'post_type' => 'qc_booking_form',
      'post_staus' => array('publish'),
      'posts_per_page' => -1
    ));

    $unacceptable_status = apply_filters('quickcab_booking_form_unacceptable_status', array(
      'auto-draft', 'future', 'draft', 'private', 'trash'
    ));

    foreach ( $post_search->posts as $booking_form ) {
      if ( in_array($booking_form->post_status, $unacceptable_status) ) {
        continue;
      }

      $booking_forms[$booking_form->ID] = new QuickCab_Booking_Form($booking_form->ID);
    }

    return $booking_forms;
  }

  public static function get_form_options() {
    $all_forms = QuickCab_Booking_Form::find_all();
    $form_options = array();

    $default_options = array(
      'form_primary_color' => quickcab_get_option('form_primary_color'),
      'form_secondary_color' => quickcab_get_option('form_secondary_color'),
      'map_starting_location' => quickcab_get_option('map_starting_location'),
      'use_fixed_addresses_starting' => quickcab_get_option('use_fixed_addresses_starting') == 'on',
      'use_fixed_addresses_destination' => quickcab_get_option('use_fixed_addresses_destination') == 'on',
      'enforce_autocomplete_restriction' => quickcab_get_option('enforce_autocompelte_country_restriction') == 'on'
    );

    if ( quickcab_get_option('enforce_autocompelte_country_restriction') == 'on' ) {
      $default_options['autocomplete_country_restriction'] = quickcab_get_option('autocomplete_country');
    }

    foreach ( $all_forms as $booking_form ) {
      $options_array = array(
        'use_fixed_addresses_starting' => $booking_form->get_fixed_address_collection(),
        'use_fixed_addresses_destination' => $booking_form->get_fixed_address_destination(),
        'enforce_autocomplete_restriction' => $booking_form->get_enforce_autocomplete_restriction(),
        'show_distance' => $booking_form->show_distance(),
        'show_duration' => $booking_form->show_duration(),
        'enable_passengers_field' => $booking_form->enable_passengers_field(),
        'enable_suitcases_field' => $booking_form->enable_suitcases_field(),
        'map' => array(
          'enable_zoom_control' => $booking_form->enable_zoom_control(),
          'enable_map_type_control' => $booking_form->enable_map_type_control(),
          'enable_street_view_control' => $booking_form->enable_street_view_control(),
          'enable_full_screen_control' => $booking_form->enable_full_screen_control(),
          'enable_pan_control' => $booking_form->enable_pan_control()
        )
      );

      if ( !empty($booking_form->get_primary_color()) ) {
        $options_array['form_primary_color'] = $booking_form->get_primary_color();
      }

      if ( !empty($booking_form->get_secondary_color()) ) {
        $options_array['form_secondary_color'] = $booking_form->get_secondary_color();
      }

      if ( !empty($booking_form->get_map_starting_location()) ) {
        $options_array['map_starting_location'] = $booking_form->get_map_starting_location();
      }

      if ( $booking_form->get_enforce_autocomplete_restriction() ) {
        $options_array['autocomplete_country_restriction'] = $booking_form->get_autocomplete_country();
      }

      $form_options[$booking_form->get_ID()] = array_merge($default_options, $options_array);
    }

    return $form_options;
  }

  public static function is_valid($post_id) {
    return get_post_type($post_id) == 'qc_booking_form' && get_post_status($post_id) == 'publish';
  }

  /**
   * Get the meta data for the booking form
   *
   * @since  1.0
   * @param  string  $key     the key to fetch
   * @param  boolean $single  get single data or array
   * @return [array|string]   value of meta data if single = true, array if not
   */
  public function get_meta($key = '', $single = true) {
    $value = get_post_meta($this->get_ID(), $this->meta_prefix . $key, $single);
    return apply_filters('quickcab_booking_form_get_meta_' . $key, $value, $this->get_ID(), $single);
  }

  /**
   * Update the meta data for the booking form
   *
   * @since  1.0
   * @param  string $key         the key to update
   * @param  string $value       the new value
   * @param  string $prev_value  the old value
   * @return [int|bool]          meta ID if key didn't exist, true on success, false on fail
   */
  public function update_meta($key = '', $value = '', $prev_value = '') {
    if ( empty($key) || $key == '' ) {
      return false;
    }

    $key = $this->meta_prefix . $key;

    $value = apply_filters('quickcab_booking_form_update_meta_' . $key, $value, $this->get_ID());
    return update_post_meta($this->get_ID(), $key, $value, $prev_value);
  }

  /**
   * Delete the meta data for the fixed address
   *
   * @since  1.0
   * @param  string $key  the key to delete
   * @return bool         success or not
   */
  public function delete_meta($key) {
    if ( empty($key) || $key == '' ) {
      return false;
    }

    $key = $this->meta_prefix . $key;

    return delete_post_meta($this->get_ID(), $key);
  }

  /**
   * Populates the default WordPress Post fields to QuickCab_Booking_Form
   *
   * @since  1.0
   * @param  WP_Post $post  the WP_Post
   * @return bool           if the save was successful
   */
  private function populate_post_fields($post) {
    if ( !is_object($post) || !is_a($post, 'WP_Post') || $post->post_type !== 'qc_booking_form' ) {
      return false;
    }

    foreach ( $post as $key => $value ) {
      $this->$key = $value;
    }

    return true;
  }

  /**
   * Setup the object variables
   *
   * @since  1.0
   * @param  WP_Post $booking_form  the booking form WP_Post
   * @return bool                     did the setup suceed?
   */
  private function setup_booking_form($booking_form) {
    if ( is_null($booking_form) || !is_a($booking_form, 'WP_Post') || $booking_form->post_type !== 'qc_booking_form' ) {
      return false;
    }

    do_action('quickcab_before_setup_booking_form', $this, $booking_form);

    $this->ID = $booking_form->ID;
    $this->wp_post = $booking_form;

    $this->primary_color                    = $this->setup_primary_color();
    $this->secondary_color                  = $this->setup_secondary_color();
    $this->map_starting_location            = $this->setup_map_starting_location();
    $this->booking_time_interval            = $this->setup_booking_time_interval();
    $this->fixed_address_collection         = $this->setup_fixed_address_collection();
    $this->fixed_address_destination        = $this->setup_fixed_address_destination();
    $this->enforce_autocomplete_restriction = $this->setup_enforce_autocomplete_restriction();
    $this->autocomplete_country             = $this->setup_autocomplete_country();
    $this->enable_extra_waiting_time        = $this->setup_enable_extra_waiting_time();
    $this->extra_waiting_time_unit          = $this->setup_extra_waiting_time_unit();
    $this->extra_waiting_time_max_value     = $this->setup_extra_waiting_time_max_value();
    $this->enable_waypoints                 = $this->setup_enable_waypoints();
    $this->enable_zoom_control              = $this->setup_enable_zoom_control();
    $this->enable_map_type_control          = $this->setup_enable_map_type_control();
    $this->enable_street_view_control       = $this->setup_enable_street_view_control();
    $this->enable_full_screen_control       = $this->setup_enable_full_screen_control();
    $this->enable_pan_control               = $this->setup_enable_pan_control();
    $this->show_distance                    = $this->setup_show_distance();
    $this->show_duration                    = $this->setup_show_duration();
    $this->enable_passengers_field          = $this->setup_enable_passengers_field();
    $this->enable_suitcases_field           = $this->setup_enable_suitcases_field();
    $this->enable_direction_field           = $this->setup_enable_direction_field();

    $this->populate_post_fields($this->wp_post);

    do_action('quickcab_after_setup_booking_form', $this, $booking_form);

    return true;
  }

  private function setup_primary_color() {
    return $this->get_meta('primary_color');
  }

  private function setup_secondary_color() {
    return $this->get_meta('secondary_color');
  }

  private function setup_map_starting_location() {
    return $this->get_meta('map_starting_location');
  }

  private function setup_booking_time_interval() {
    return intval($this->get_meta('booking_time_interval'));
  }

  private function setup_fixed_address_collection() {
    return (bool) $this->get_meta('fixed_address_collection');
  }

  private function setup_fixed_address_destination() {
    return (bool) $this->get_meta('fixed_address_destination');
  }

  private function setup_enforce_autocomplete_restriction() {
    return (bool) $this->get_meta('enforce_autocomplete_restriction');
  }

  private function setup_autocomplete_country() {
    return $this->get_meta('autocomplete_country');
  }

  private function setup_enable_extra_waiting_time() {
    return (bool) $this->get_meta('enable_extra_waiting_time');
  }

  private function setup_extra_waiting_time_unit() {
    return $this->get_meta('extra_waiting_time_unit');
  }

  private function setup_extra_waiting_time_max_value() {
    return intval($this->get_meta('extra_waiting_time_max_value'));
  }

  private function setup_enable_waypoints() {
    return (bool) $this->get_meta('enable_waypoints');
  }

  private function setup_enable_zoom_control() {
    return (bool) $this->get_meta('enable_zoom_control');
  }

  private function setup_enable_map_type_control() {
    return (bool) $this->get_meta('enable_map_type_control');
  }

  private function setup_enable_street_view_control() {
    return (bool) $this->get_meta('enable_street_view_control');
  }

  private function setup_enable_full_screen_control() {
    return (bool) $this->get_meta('enable_full_screen_control');
  }

  private function setup_enable_pan_control() {
    return (bool) $this->get_meta('enable_pan_control');
  }

  private function setup_show_distance() {
    return (bool) $this->get_meta('show_distance');
  }

  private function setup_show_duration() {
    return (bool) $this->get_meta('show_duration');
  }

  private function setup_enable_passengers_field() {
    return (bool) $this->get_meta('enable_passengers_field');
  }

  private function setup_enable_suitcases_field() {
    return (bool) $this->get_meta('enable_suitcases_field');
  }

  private function setup_enable_direction_field() {
    return (bool) $this->get_meta('enable_direction_field');
  }
}
