<?php

if ( !defined('ABSPATH') ) exit;

function quickcab_render_geofence_metabox_price_row($name, $post_id, $label_options = array(), $input_options = array(), $helper_options = array()) {
  $meta_name = str_replace('quickcab_geofence_', '', $name);
  $geofence_rule = new QuickCab_Geofence_Rule($post_id);

  if ( !isset($input_options['value']) ) {
    $input_options['value'] = $geofence_rule->get_meta($meta_name);
  }

  quickcab_render_metabox_price_row($name, $post_id, $label_options, $input_options, $helper_options);
}

function quickcab_render_geofence_metabox_string_row($name, $post_id, $label_options = array(), $input_options = array(), $helper_options = array()) {
  $meta_name = str_replace('quickcab_geofence_', '', $name);
  $geofence_rule = new QuickCab_Geofence_Rule($post_id);

  if ( !isset($input_options['value']) ) {
    $input_options['value'] = $geofence_rule->get_meta($meta_name);
  }

  quickcab_render_metabox_string_row($name, $post_id, $label_options, $input_options, $helper_options);
}

function quickcab_render_geofence_metabox_checkbox_row($name, $post_id, $label_options = array(), $input_options = array(), $helper_options = array()) {
  $meta_name = str_replace('quickcab_geofence_', '', $name);
  $geofence_rule = new QuickCab_Geofence_Rule($post_id);

  if ( !isset($input_options['checked']) ) {
    $input_options['checked'] = (bool) $geofence_rule->get_meta($meta_name);
  }

  quickcab_render_metabox_checkbox_row($name, $post_id, $label_options, $input_options, $helper_options);
}

/**
 * Determines whether the given coordinates falls within a given polygon by
 * determining how many times the coordinate crosses the outer edges of the
 * polygon
 *
 * @since  1.0
 * @param  array   $poly_coords  array of coordinates of the polygon
 * @param  array   $destination  array containing the 'lat' and 'long' of the coordinates
 * @return boolean               is the point in the polygon?
 */
function quickcab_is_point_in_polygon($poly_coords, $destination) {
  $polygon_no_sides = count($poly_coords) - 1;
  $j = $polygon_no_sides - 1;
  $odd_nodes = false;

  for ( $i = 0; $i < $polygon_no_sides; $i++ ) {
    if (
      (
        // if destination lat is above one and below (or equal to) another
        // consecutive point in the polygon (or vice versa)
        ($poly_coords[$i]['lat'] < $destination['lat'] && $poly_coords[$j]['lat'] >= $destination['lat']) ||
        ($poly_coords[$j]['lat'] < $destination['lat'] && $poly_coords[$i]['lat'] >= $destination['lat'])
      )
      &&
      (
        // if its further than the line
        (
          $poly_coords[$i]['long'] + ($destination['lat'] - $poly_coords[$i]['lat']) /
          ($poly_coords[$j]['lat'] - $poly_coords[$i]['lat']) * ($poly_coords[$j]['long'] - $poly_coords[$i]['long'])
        )
        <
        $destination['lng']
      )
    ) {
      $odd_nodes = !$odd_nodes;
    }

    $j = $i;
  }

  return $odd_nodes;
}

/**
 * Turns an array into a string representation to store in DB
 *
 * @since  1.0
 * @param  array  $vehicle_ids  array of vehicle IDs
 * @return string
 */
function quickcab_format_geofence_vehicles_parameter($vehicle_ids, $post_id) {
  return implode(",", $vehicle_ids);
}
add_filter('quickcab_geofence_metabox_save_vehicle_ids', 'quickcab_format_geofence_vehicles_parameter', 10, 2);

/**
 * Adds any additional pricing to the price inof object when a hote is made
 *
 * @since  1.0
 * @param  array      $price_info  the $price_info array from the quote
 * @param  QuickCab_Vehicle $vehicle     the vehicle being quoted
 * @param  QuickCab_Journey $journey     the journey being quoted
 * @return array                   the new $price_info array
 */
function quickcab_geofence_calculate_price($price_info, $vehicle, $journey) {
  $geofence_rules = QuickCab_Geofence_Rule::find_all();

  // This needs to be run 2nd to last as it holds the rules with fixed prices
  // The one-way/return check is run last
  foreach ( $geofence_rules as $geofence_rule ) {

    // Don't test if rule doesn't apply to current vehicle
    if ( !$geofence_rule->rule_applies_to($vehicle->get_ID()) ) {
      continue;
    }

    if ( $geofence_rule->is_one_way() ) {

      $collection_ok = $geofence_rule->starting_area_anywhere() || quickcab_is_point_in_polygon($geofence_rule->get_starting_coords(), $journey->get_origin_coords());
      $destination_ok = $geofence_rule->destination_area_anywhere() || quickcab_is_point_in_polygon($geofence_rule->get_destination_coords(), $journey->get_destination_coords());

      // Customer is picked up within defined area for condition
      // AND
      // Customer is dropped off within defined area for condition
      if ( $collection_ok && $destination_ok ) {
        if ( $geofence_rule->is_fixed_price() ) {
          foreach ( array_keys($price_info['breakdown']) as $breakdown_item ) {
            if ( $breakdown_item !== 'surge_pricing' || $geofence_rule->ignore_surge_pricing_rule() ) {
              unset($price_info['breakdown'][$breakdown_item]);
            }
          }

          $price_info = quikcab_add_to_price_info(
            $price_info,
            'geofence_charge',
            esc_html__('Fixed Pricing Rule', 'quickcab'),
            $geofence_rule->get_fixed_price_amount(),
            ''
          );
        } elseif ( $geofence_rule->is_increased_price() ) {
          $price_info = quikcab_add_to_price_info(
            $price_info,
            'geofence_charge',
            esc_html__('Fixed Pricing Rule', 'quickcab'),
            $geofence_rule->get_increase_price_amount(),
            ''
          );
        }
      }
    } elseif ( $geofence_rule->is_both_ways() ) {

      $a_to_b_collection_ok = $geofence_rule->starting_area_anywhere() || quickcab_is_point_in_polygon($geofence_rule->get_starting_coords(), $journey->get_origin_coords());
      $a_to_b_destination_ok = $geofence_rule->destination_area_anywhere() || quickcab_is_point_in_polygon($geofence_rule->get_destination_coords(), $journey->get_destination_coords());

      $b_to_a_collection_ok = $geofence_rule->destination_area_anywhere() || quickcab_is_point_in_polygon($geofence_rule->get_destination_coords(), $journey->get_origin_coords());
      $b_to_a_destination_ok = $geofence_rule->starting_area_anywhere() || quickcab_is_point_in_polygon($geofence_rule->get_starting_coords(), $journey->get_destination_coords());

      //   Customer is picked up within defined pickup area for condition
      //   AND
      //   Customer is dropped off within defined dropoff area for condition
      // OR
      //   Customer is picked up within defined dropoff area for condition
      //   AND
      //   Customer is dropped off within defined pickup area for condition
      if (
        ( $a_to_b_collection_ok && $a_to_b_destination_ok )
        ||
        ( $b_to_a_collection_ok && $b_to_a_destination_ok )
      ) {

        if ( $geofence_rule->is_fixed_price() ) {
          foreach ( array_keys($price_info['breakdown']) as $breakdown_item ) {
            if ( $breakdown_item !== 'surge_pricing' || $geofence_rule->ignore_surge_pricing_rule() ) {
              unset($price_info['breakdown'][$breakdown_item]);
            }
          }

          $price_info = quikcab_add_to_price_info(
            $price_info,
            'geofence_charge',
            esc_html__('Fixed Pricing Rule', 'quickcab'),
            $geofence_rule->get_fixed_price_amount(),
            ''
          );
        } elseif ( $geofence_rule->is_increased_price() ) {
          $price_info = quikcab_add_to_price_info(
            $price_info,
            'geofence_charge',
            esc_html__('Fixed Pricing Rule', 'quickcab'),
            $geofence_rule->get_increase_price_amount(),
            ''
          );
        }
      }
    }
  }

  return apply_filters('quickcab_geofence_total_fare', $price_info, $vehicle, $journey);
}
add_filter('quickcab_price_after_all_vehicle_costs_calculation', 'quickcab_geofence_calculate_price', 11, 3);
