<?php

if ( !defined('ABSPATH') ) exit;

function quickcab_woocommerce_checkout_fields($checkout) {
  global $woocommerce;

  if ( !quickcab_product_in_cart() ) {
    return;
  }

  $booking_form = new QuickCab_Booking_Form($woocommerce->session->get('quickcab_form_id'));

  ?><input type="hidden" name="quickcab_checkout" value="1" /><?php

  ?><div class="form-row my-field-class form-row-wide">
    <label>
      <strong><?php
        echo esc_html__('Starting Address', 'quickcab') . ':';
      ?></strong>
    </label>
    <div class="col-sm-7">
      <p class="static-form-control"><?php
        echo esc_html($woocommerce->session->get('quickcab_starting_address'));
      ?></p>
    </div>
  </div><?php

  if ( $woocommerce->session->get('quickcab_uses_waypoints') ) {
    $waypoint_count = intval($woocommerce->session->get('quickcab_waypoints_count'));

    ?><div class="form-row my-field-class form-row-wide">
      <label>
        <strong><?php
          echo esc_html__('Waypoints', 'quickcab') . ':';
        ?></strong>
      </label>
      <div class="col-sm-7">
        <p class="static-form-control">
          <ul><?php
            for ( $i = 0; $i < $waypoint_count; $i++ ) {
              ?><li><?php
                echo esc_html($woocommerce->session->get('quickcab_waypoint_' . $i . '_address'));
              ?></li><?php
            }
          ?></ul>
        </p>
      </div>
    </div><?php
  }

  ?><div class="form-row my-field-class form-row-wide">
    <label>
      <strong><?php
        echo esc_html__('Destination Address', 'quickcab') . ':';
      ?></strong>
    </label>
    <div class="col-sm-7">
      <p class="static-form-control"><?php
        echo esc_html($woocommerce->session->get('quickcab_destination_address'));
      ?></p>
    </div>
  </div><?php

  ?><div class="form-row my-field-class form-row-wide">
    <label>
      <strong><?php
        echo esc_html__('Departure Time', 'quickcab') . ':';
      ?></strong>
    </label>
    <div class="col-sm-7">
      <p class="static-form-control"><?php
        echo date("h:ia " . get_option('date_format'), strtotime($woocommerce->session->get('quickcab_departure_time')));
      ?></p>
    </div>
  </div><?php

  if ( $woocommerce->session->get('quickcab_is_return') ) {
    ?><div class="form-row my-field-class form-row-wide">
      <label>
        <strong><?php
          echo esc_html__('Return Departure Time', 'quickcab') . ':';
        ?></strong>
      </label>
      <div class="col-sm-7">
        <p class="static-form-control"><?php
          echo date("h:ia " . get_option('date_format'), strtotime($woocommerce->session->get('quickcab_return_departure_time')));
        ?></p>
      </div>
    </div><?php
  }

  ?><div class="form-row my-field-class form-row-wide">
    <label>
      <strong><?php
        echo esc_html__('Vehicle', 'quickcab') . ':';
      ?></strong>
    </label>
    <div class="col-sm-7">
      <p class="static-form-control"><?php
        $vehicle_id = intval($woocommerce->session->get('quickcab_vehicle_id'));
        $vehicle = new QuickCab_Vehicle($vehicle_id);

        echo esc_html($vehicle->get_name());
      ?></p>
    </div>
  </div><?php

  if ( $booking_form->enable_passengers_field() ) {
    ?><div class="form-row my-field-class form-row-wide">
      <label>
        <strong><?php
          echo esc_html__('Occupants', 'quickcab') . ':';
        ?></strong>
      </label>
      <div class="col-sm-7">
        <p class="static-form-control"><?php
          echo esc_html($woocommerce->session->get('quickcab_occupants'));
        ?></p>
      </div>
    </div><?php
  }

  if ( $booking_form->enable_suitcases_field() ) {
    ?><div class="form-row my-field-class form-row-wide">
      <label>
        <strong><?php
          echo esc_html__('Suitcases', 'quickcab') . ':';
        ?></strong>
      </label>
      <div class="col-sm-7">
        <p class="static-form-control"><?php
          echo esc_html($woocommerce->session->get('quickcab_suitcases'));
        ?></p>
      </div>
    </div><?php
  }

  if ( $woocommerce->session->get('waiting_time') ) {
    ?><div class="form-row my-field-class form-row-wide">
      <label>
        <strong><?php
          echo esc_html__('Waiting Time', 'quickcab') . ':';
        ?></strong>
      </label>
      <div class="col-sm-7">
        <p class="static-form-control"><?php
          $waiting_time = $woocommerce->session->get('waiting_time');
          echo $waiting_time . ' ' . ($waiting_time == 1 ? esc_html__('hour', 'quickcab') : esc_html__('hours', 'quickcab'));
        ?></p>
      </div>
    </div><?php
  }
}
add_filter('woocommerce_after_order_notes', 'quickcab_woocommerce_checkout_fields');

function quickcab_update_order_with_journey_info($order_id) {
  global $woocommerce;

  if ( !isset($_POST['quickcab_checkout']) ) {
    return;
  }

  $order = new QuickCab_Order($order_id);
  $booking_form = new QuickCab_Booking_Form($woocommerce->session->get('quickcab_form_id'));

  if ( $booking_form->enable_passengers_field() ) {
    $order->update_meta('occupants', $woocommerce->session->get('quickcab_occupants'));
  }

  if ( $booking_form->enable_suitcases_field() ) {
    $order->update_meta('suitcases', $woocommerce->session->get('quickcab_suitcases'));
  }

  if ( $booking_form->enable_extra_waiting_time() ) {
    $order->update_meta('waiting_time', $woocommerce->session->get('quickcab_waiting_time'));
  }

  $order->update_meta('vehicle_id', $woocommerce->session->get('quickcab_vehicle_id'));
  $order->update_meta('departure_time', $woocommerce->session->get('quickcab_departure_time'));
  $order->update_meta('is_return', $woocommerce->session->get('quickcab_is_return'));
  $order->update_meta('notes', $woocommerce->session->get('quickcab_notes'));
  $order->update_meta('duration_in_seconds', $woocommerce->session->get('quickcab_duration_in_seconds'));
  $order->update_meta('duration_in_words', $woocommerce->session->get('quickcab_duration_in_words'));
  $order->update_meta('duration_in_seconds_traffic', $woocommerce->session->get('quickcab_duration_in_seconds_traffic'));
  $order->update_meta('duration_in_words_traffic', $woocommerce->session->get('quickcab_duration_in_words_traffic'));
  $order->update_meta('distance_in_metres', $woocommerce->session->get('quickcab_distance_in_metres'));
  $order->update_meta('distance_in_words', $woocommerce->session->get('quickcab_distance_in_words'));
  $order->update_meta('starting_lat', $woocommerce->session->get('quickcab_starting_coords_lat'));
  $order->update_meta('starting_long', $woocommerce->session->get('quickcab_starting_coords_long'));
  $order->update_meta('starting_address', $woocommerce->session->get('quickcab_starting_address'));
  $order->update_meta('destination_lat', $woocommerce->session->get('quickcab_destination_coords_lat'));
  $order->update_meta('destination_long', $woocommerce->session->get('quickcab_destination_coords_long'));
  $order->update_meta('destination_address', $woocommerce->session->get('quickcab_destination_address'));
  $order->update_meta('form_id', $woocommerce->session->get('quickcab_form_id'));

  quickcab_set_base_transit_times($order_id);

  if ( $woocommerce->session->get('quickcab_is_return') ) {
    $order->update_meta('return_departure_time', $woocommerce->session->get('quickcab_return_departure_time'));
    $order->update_meta('return_duration_in_seconds', $woocommerce->session->get('quickcab_return_duration_in_seconds'));
    $order->update_meta('return_duration_in_words', $woocommerce->session->get('quickcab_return_duration_in_words'));
    $order->update_meta('return_duration_in_seconds_traffic', $woocommerce->session->get('quickcab_return_duration_in_seconds_traffic'));
    $order->update_meta('return_duration_in_words_traffic', $woocommerce->session->get('quickcab_return_duration_in_words_traffic'));
  }

  if ( $woocommerce->session->get('quickcab_uses_waypoints') ) {
    $waypoints_count = intval($woocommerce->session->get('quickcab_waypoints_count'));

    $order->update_meta('waypoints_count', $waypoints_count);

    for ( $i = 0; $i < $waypoints_count; $i++ ) {
      $order->update_meta('waypoint_' . $i . '_lat', $woocommerce->session->get('quickcab_waypoint_' . $i . '_lat'));
      $order->update_meta('waypoint_' . $i . '_lng', $woocommerce->session->get('quickcab_waypoint_' . $i . '_lng'));
      $order->update_meta('waypoint_' . $i . '_address', $woocommerce->session->get('quickcab_waypoint_' . $i . '_address'));
    }
  }

  $custom_fields = QuickCab_Checkout_Field::find_all();

  $unique_keys = array();

  foreach ( $custom_fields as $custom_field ) {
    $order->update_meta('custom_checkout_field_label_' . $custom_field->get_unique_key(), $custom_field->get_label());
    $order->update_meta('custom_checkout_field_' . $custom_field->get_unique_key(), $_POST[$custom_field->get_unique_key()]);
    $order->update_meta('custom_checkout_field_show_email_' . $custom_field->get_unique_key(), $custom_field->show_in_email());
    array_push($unique_keys, $custom_field->get_unique_key());
  }

  $order->update_meta('custom_checkout_field_keys', serialize($unique_keys));
}
add_action('woocommerce_checkout_update_order_meta', 'quickcab_update_order_with_journey_info');

/**
 * Calculate the times to transit from base locaton to collection point and from
 * destination to base location. This is used to determine if a vehicle is available
 * when a user requests a quote. This is done here so we do not have to compute it
 * every time the user requests a quote.
 */
function quickcab_set_base_transit_times($post_id) {
  if ( quickcab_get_option('use_base_location_for_availability') !== 'on' ) {
    return;
  }

  $order = new QuickCab_Order($post_id);
  $base_location = quickcab_get_option('vehicle_base_location');
  $results = quickcab_calculate_base_transit_buffer_times($post_id);

  if ( $results['base_to_collection'] && $results['destination_to_base'] ) {
    $order->update_meta('base_location', $base_location);

    $order->update_meta('base_to_collection_time', $results['base_to_collection']);
    $order->update_meta('destination_to_base_time', $results['destination_to_base']);
  }

  return $results;
}

function quickcab_product_in_cart() {
  global $woocommerce;

  foreach ( $woocommerce->cart->get_cart() as $cart_item_key => $values ) {
    $product = $values['data'];

    if ( $product->get_id() == quickcab_get_wc_product() ) {
      return true;
    }
  }

  return false;
}

function quickcab_email_order_receipt($order, $sent_to_admin) {
  $quickcab_order = new QuickCab_Order($order->get_id());

  if ( !$quickcab_order->get_form_id() ) {
    return;
  }

  $fields = array(
    'starting_address' => esc_html__('Starting Address', 'quickcab'),
    'destination_address' => esc_html__('Destination Address', 'quickcab'),
    'pick_up_time' => esc_html__('Departure Time', 'quickcab')
  );

  ?><img src="<?php echo esc_attr($quickcab_order->get_static_map_url()); ?>">

  <table class="td" cellspacing="0" cellpadding="6" style="width: 100%; font-family: 'Helvetica Neue', Helvetica, Roboto, Arial, sans-serif; color: #737373; border: 1px solid #e4e4e4; margin-top: 20px; margin-bottom: 20px;" border="1">
    <thead>
      <tr>
        <th class="td" scope="col" style="text-align: left; color: #737373; border: 1px solid #e4e4e4; padding: 12px;"><?php
          echo esc_html__('Field', 'quickcab');
        ?></th>
        <th class="td" scope="col" style="text-align: left; color: #737373; border: 1px solid #e4e4e4; padding: 12px;"><?php
          echo esc_html__('Value', 'quickcab');
        ?></th>
      </tr>
    </thead>
    <tbody>
      <tr class="order_item">
        <td class="td" style="text-align: left; vertical-align: middle; border: 1px solid #eee; font-family: 'Helvetica Neue', Helvetica, Roboto, Arial, sans-serif; word-wrap: break-word; color: #737373; padding: 12px;"><?php
          echo esc_html__('Vehicle', 'quickcab');
        ?></td>
        <td class="td" style="text-align: left; vertical-align: middle; border: 1px solid #eee; font-family: 'Helvetica Neue', Helvetica, Roboto, Arial, sans-serif; color: #737373; padding: 12px;"><?php
          echo esc_html($quickcab_order->get_vehicle_name());
        ?></td>
      </tr>
      <tr class="order_item">
        <td class="td" style="text-align: left; vertical-align: middle; border: 1px solid #eee; font-family: 'Helvetica Neue', Helvetica, Roboto, Arial, sans-serif; word-wrap: break-word; color: #737373; padding: 12px;"><?php
          echo esc_html__('Collection Address', 'quickcab');
        ?></td>
        <td class="td" style="text-align: left; vertical-align: middle; border: 1px solid #eee; font-family: 'Helvetica Neue', Helvetica, Roboto, Arial, sans-serif; color: #737373; padding: 12px;"><?php
          echo esc_html($quickcab_order->get_starting_address());
        ?></td>
      </tr><?php

      $waypoints = $quickcab_order->get_waypoints();
      if ( count($waypoints) > 0 ) {
        ?><tr class="order_item">
          <td class="td" style="text-align: left; vertical-align: middle; border: 1px solid #eee; font-family: 'Helvetica Neue', Helvetica, Roboto, Arial, sans-serif; word-wrap: break-word; color: #737373; padding: 12px;"><?php
            echo esc_html__('Waypoints', 'quickcab');
          ?></td>
          <td class="td" style="text-align: left; vertical-align: middle; border: 1px solid #eee; font-family: 'Helvetica Neue', Helvetica, Roboto, Arial, sans-serif; color: #737373; padding: 12px;"><?php
            foreach ( $waypoints as $waypoint ) {
              echo $waypoint['address'] . '<br>';
            }
          ?></td>
        </tr><?php
      }

      ?><tr class="order_item">
        <td class="td" style="text-align: left; vertical-align: middle; border: 1px solid #eee; font-family: 'Helvetica Neue', Helvetica, Roboto, Arial, sans-serif; word-wrap: break-word; color: #737373; padding: 12px;"><?php
          echo esc_html__('Destination Address', 'quickcab');
        ?></td>
        <td class="td" style="text-align: left; vertical-align: middle; border: 1px solid #eee; font-family: 'Helvetica Neue', Helvetica, Roboto, Arial, sans-serif; color: #737373; padding: 12px;"><?php
          echo esc_html($quickcab_order->get_destination_address());
        ?></td>
      </tr>
      <tr class="order_item">
        <td class="td" style="text-align: left; vertical-align: middle; border: 1px solid #eee; font-family: 'Helvetica Neue', Helvetica, Roboto, Arial, sans-serif; word-wrap: break-word; color: #737373; padding: 12px;"><?php
          echo esc_html__('One Way', 'quickcab') . '/' . esc_html__('Return', 'quickcab');
        ?></td>
        <td class="td" style="text-align: left; vertical-align: middle; border: 1px solid #eee; font-family: 'Helvetica Neue', Helvetica, Roboto, Arial, sans-serif; color: #737373; padding: 12px;"><?php
          echo $quickcab_order->is_return() ? esc_html('Return', 'quickcab') : esc_html('One Way', 'quickcab');
        ?></td>
      </tr>
      <tr class="order_item">
        <td class="td" style="text-align: left; vertical-align: middle; border: 1px solid #eee; font-family: 'Helvetica Neue', Helvetica, Roboto, Arial, sans-serif; word-wrap: break-word; color: #737373; padding: 12px;"><?php
          echo esc_html__('Departure Time', 'quickcab');
        ?></td>
        <td class="td" style="text-align: left; vertical-align: middle; border: 1px solid #eee; font-family: 'Helvetica Neue', Helvetica, Roboto, Arial, sans-serif; color: #737373; padding: 12px;"><?php
          echo esc_html($quickcab_order->get_departure_time_in_format());
        ?></td>
      </tr><?php
      if ( $quickcab_order->is_return() ) {
        ?><tr class="order_item">
          <td class="td" style="text-align: left; vertical-align: middle; border: 1px solid #eee; font-family: 'Helvetica Neue', Helvetica, Roboto, Arial, sans-serif; word-wrap: break-word; color: #737373; padding: 12px;"><?php
            echo esc_html__('Return Departure Time', 'quickcab');
          ?></td>
          <td class="td" style="text-align: left; vertical-align: middle; border: 1px solid #eee; font-family: 'Helvetica Neue', Helvetica, Roboto, Arial, sans-serif; color: #737373; padding: 12px;"><?php
            echo esc_html($quickcab_order->get_return_departure_time_in_format());
          ?></td>
        </tr><?php
      }
      ?><tr class="order_item">
        <td class="td" style="text-align: left; vertical-align: middle; border: 1px solid #eee; font-family: 'Helvetica Neue', Helvetica, Roboto, Arial, sans-serif; word-wrap: break-word; color: #737373; padding: 12px;"><?php
          echo esc_html__('Passengers', 'quickcab');
        ?></td>
        <td class="td" style="text-align: left; vertical-align: middle; border: 1px solid #eee; font-family: 'Helvetica Neue', Helvetica, Roboto, Arial, sans-serif; color: #737373; padding: 12px;"><?php
          echo esc_html($quickcab_order->get_occupants());
        ?></td>
      </tr>
      <tr class="order_item">
        <td class="td" style="text-align: left; vertical-align: middle; border: 1px solid #eee; font-family: 'Helvetica Neue', Helvetica, Roboto, Arial, sans-serif; word-wrap: break-word; color: #737373; padding: 12px;"><?php
          echo esc_html__('Suitcases', 'quickcab');
        ?></td>
        <td class="td" style="text-align: left; vertical-align: middle; border: 1px solid #eee; font-family: 'Helvetica Neue', Helvetica, Roboto, Arial, sans-serif; color: #737373; padding: 12px;"><?php
          echo esc_html($quickcab_order->get_suitcases());
        ?></td>
      </tr><?php

      foreach ( $quickcab_order->get_custom_checkout_fields() as $checkout_field ) {
        if ( $checkout_field['show_in_email'] ) {
          ?><tr class="order_item">
            <td class="td" style="text-align: left; vertical-align: middle; border: 1px solid #eee; font-family: 'Helvetica Neue', Helvetica, Roboto, Arial, sans-serif; word-wrap: break-word; color: #737373; padding: 12px;"><?php
              echo $checkout_field['label'];
            ?></td>
            <td class="td" style="text-align: left; vertical-align: middle; border: 1px solid #eee; font-family: 'Helvetica Neue', Helvetica, Roboto, Arial, sans-serif; color: #737373; padding: 12px;"><?php
              echo $checkout_field['value'];
            ?></td>
          </tr><?php
        }
      }

    ?></tbody>
  </table><?php
}
add_action('woocommerce_email_before_order_table', 'quickcab_email_order_receipt', 10, 2);

function quickcab_customise_checkout_fields($wc_fields) {
  if ( quickcab_product_in_cart() ) {
    $custom_fields = new WP_Query(array(
      'post_type'      => 'qc_checkout_field',
      'post_status'    => 'publish',
      'posts_per_page' => '-1'
    ));

    while ( $custom_fields->have_posts() ) {
      $custom_fields->the_post();
      $custom_field = new QuickCab_Checkout_Field(get_the_ID());

      if ( empty($custom_field->get_unique_key()) ) {
        continue;
      }

      $input_options = array(
        'label' => $custom_field->get_label(),
        'placeholder' => $custom_field->get_placeholder(),
        'required' => $custom_field->is_required(),
        'class' => array(),
        'clear' => true,
        'type' => $custom_field->get_input_type()
      );

      if ( $custom_field->get_input_type() == 'select' ) {
        $input_options['options'] = $custom_field->get_input_options_woocommerce();
      }

      $wc_fields['order'][$custom_field->get_unique_key()] = $input_options;
    }
  }

  return $wc_fields;
}
add_filter('woocommerce_checkout_fields', 'quickcab_customise_checkout_fields');

function quickcab_get_wc_product() {
  $existing_id = quickcab_get_option('wc_product_id');

  $unacceptable_status = array('auto-draft', 'future', 'draft', 'private', 'trash');

  if (
    !empty($existing_id) &&
    !is_null(get_post($existing_id)) &&
    !in_array(get_post_status($existing_id), $unacceptable_status)
  ) {
    return $existing_id;
  }

  return -1;
}

/**
 * If a WooCommerce product already exists return its ID, else create a new one
 *
 * @since   1.0
 * @return  int      ID of the WooCommerce product
 */
function quickcab_get_or_create_wc_product() {

  $fetched_id = quickcab_get_wc_product();

  if ( $fetched_id != -1 ) {
    return $fetched_id;
  }

  $user_id = get_current_user_id() ? get_current_user_id() : 1;

  $new_product = array(
    'post_author' => $user_id,
    'post_title' => 'Journey',
    'post_status' => 'publish',
    'post_type' => 'product'
  );

  $post_id = wp_insert_post($new_product);

  quickcab_set_option('wc_product_id', $post_id);

  update_post_meta($post_id, '_stock_status', 'instock');
  update_post_meta($post_id, '_visibility', 'hidden');
  update_post_meta($post_id, '_stock', '');
  update_post_meta($post_id, '_virtual', 'yes');
  update_post_meta($post_id, '_regular_price', '0');
  update_post_meta($post_id, '_featured', 'no');
  update_post_meta($post_id, '_manage_stock', 'no');
  update_post_meta($post_id, '_sold_individually', 'yes');
  update_post_meta($post_id, '_price', '0');

  return $post_id;
}
