<?php

if ( !defined('ABSPATH') ) exit;

/**
 * Register the custom post types for QuickCab
 *
 * @since  1.0
 * @return void
 */
function quickcab_register_custom_post_types() {
  quickcab_register_vehicles_post_type();
  quickcab_register_vehicle_features_taxonomy();
  quickcab_register_booking_form_post_type();
  quickcab_register_geofence_post_type();
  quickcab_register_fixed_addresses_post_type();
  quickcab_register_surge_price_post_type();
  quickcab_register_checkout_field_post_type();
}
add_action('init', 'quickcab_register_custom_post_types');

function quickcab_register_vehicles_post_type() {
  $vehicle_labels = array(
    'name'                  => '%2$s',
    'singular_name'         => '%1$s',
    'add_new'               => esc_html__('Add New %1$s', 'quickcab'),
    'add_new_item'          => esc_html__('Add New %1$s', 'quickcab'),
    'edit_item'             => esc_html__('Edit %1$s', 'quickcab'),
    'new_item'              => esc_html__('New %1$s', 'quickcab'),
    'all_items'             => esc_html__('All %2$s', 'quickcab'),
    'view_item'             => esc_html__('View %1$s', 'quickcab'),
    'search_items'          => esc_html__('Search %2$s', 'quickcab'),
    'not_found'             => esc_html__('No %2$s found', 'quickcab'),
    'not_found_in_trash'    => esc_html__('No %2$s found in Trash', 'quickcab'),
    'parent_item_colon'     => '',
    'menu_name'             => 'QuickCab',
    'featured_image'        => esc_html__('%1$s Image', 'quickcab'),
    'set_featured_image'    => esc_html__('Set %1$s Image', 'quickcab'),
    'remove_featured_image' => esc_html__('Remove %1$s Image', 'quickcab'),
    'use_featured_image'    => esc_html__('Use as %1$s Image', 'quickcab'),
    'filter_items_list'     => esc_html__('Filter %2$s list', 'quickcab'),
    'items_list_navigation' => esc_html__('%2$s list navigation', 'quickcab'),
    'items_list'            => esc_html__('%2$s list', 'quickcab'),
  );

  $singular_name = esc_html__('Vehicle', 'quickcab');
  $plural_name = esc_html__('Vehicles', 'quickcab');

  foreach ($vehicle_labels as $key => $value) {
    $vehicle_labels[$key] = sprintf($value, $singular_name, $plural_name);
  }

  $vehicle_post_type_args = array(
    'labels'             => apply_filters('quickcab_vehicle_post_type_labels', $vehicle_labels),
    'public'             => false,
    'publicly_queryable' => true,
    'show_ui'            => true,
    'show_in_menu'       => true,
    'query_var'          => true,
    'menu_icon'          => 'dashicons-dashboard',
    'map_meta_cap'       => true,
    'has_archive'        => true,
    'hierarchical'       => false,
    'supports'           => apply_filters('quickcab_vehicle_supports', array('title', 'thumbnail'))
  );

  register_post_type(
    'quickcab_vehicle',
    apply_filters('quickcab_vehicle_post_type_args', $vehicle_post_type_args)
  );
}

function quickcab_register_vehicle_features_taxonomy() {
  $vehicle_features_labels = array(
    'name'              => '%2$s',
    'singular_name'     => '%1$s',
    'search_items'      => esc_html__('Search %2$s', 'quickcab'),
    'all_items'         => esc_html__('All %1$s', 'quickcab'),
    'parent_item'       => esc_html__('Parent %1$s', 'quickcab'),
    'parent_item_colon' => esc_html__('Parent %1$s', 'quickcab') . ':',
    'edit_item'         => esc_html__('Edit %1$s', 'quickcab'),
    'update_item'       => esc_html__('Update %1$s', 'quickcab'),
    'add_new_item'      => esc_html__('Add New %1$s', 'quickcab'),
    'new_item_name'     => esc_html__('New %1$s Name', 'quickcab'),
    'menu_name'         => esc_html__('%2$s', 'quickcab'),
  );

  $singular_name = esc_html__('Vehicle Feature', 'quickcab');
  $plural_name = esc_html__('Vehicle Features', 'quickcab');

  foreach ($vehicle_features_labels as $key => $value) {
    $vehicle_features_labels[$key] = sprintf($value, $singular_name, $plural_name);
  }

  $vehicle_features_args = array(
    'hierarchical' => false,
    'labels'       => apply_filters('quickcab_vehicle_features_taxonomy_labels', $vehicle_features_labels),
    'show_ui'      => true,
    'query_var'    => 'vehicle_features',
    'rewrite'      => array(
      'slug'         => 'quickcab_vehicles/feature',
      'with_front'   => false,
      'hierarchical' => true
    )
  );

  register_taxonomy(
    'vehicle_features',
    array('quickcab_vehicle'),
    apply_filters('quickcab_vehicle_features_taxonomy_args', $vehicle_features_args)
  );
  register_taxonomy_for_object_type('vehicle_features', 'quickcab_vehicle');
}

/**
 * Change the placeholder of the title field for new posts
 *
 * @since  1.0
 * @param  string $title  the original title
 * @return string         the new title
 */
function quickcab_change_default_title($title) {
  $screen = get_current_screen();

  switch ( $screen->post_type ) {
    case 'quickcab_vehicle':
      $title = esc_html__('Enter vehicle name here', 'quickcab');
      break;
    case 'geofence_rule':
      $title = esc_html__('Enter geofence rule name here', 'quickcab');
      break;
  }

  return $title;
}
add_filter('enter_title_here', 'quickcab_change_default_title');

function quickcab_register_geofence_post_type() {
  $geofence_rule_labels = array(
    'name'                  => '%2$s',
    'singular_name'         => '%1$s',
    'add_new'               => esc_html__('Add New', 'quickcab'),
    'add_new_item'          => esc_html__('Add New %1$s', 'quickcab'),
    'edit_item'             => esc_html__('Edit %1$s', 'quickcab'),
    'new_item'              => esc_html__('New %1$s', 'quickcab'),
    'all_items'             => esc_html__('%2$s', 'quickcab'),
    'view_item'             => esc_html__('View %1$s', 'quickcab'),
    'search_items'          => esc_html__('Search %2$s', 'quickcab'),
    'not_found'             => esc_html__('No %2$s found', 'quickcab'),
    'not_found_in_trash'    => esc_html__('No %2$s found in Trash', 'quickcab'),
    'parent_item_colon'     => '',
    'menu_name'             => _x('%2$s', 'vehicle post type menu name', 'quickcab'),
    'featured_image'        => esc_html__('%1$s Image', 'quickcab'),
    'set_featured_image'    => esc_html__('Set %1$s Image', 'quickcab'),
    'remove_featured_image' => esc_html__('Remove %1$s Image', 'quickcab'),
    'use_featured_image'    => esc_html__('Use as %1$s Image', 'quickcab'),
    'filter_items_list'     => esc_html__('Filter %2$s list', 'quickcab'),
    'items_list_navigation' => esc_html__('%2$s list navigation', 'quickcab'),
    'items_list'            => esc_html__('%2$s list', 'quickcab'),
  );

  $singular_name = esc_html__('Geofence Rule', 'quickcab');
  $plural_name = esc_html__('Geofence Rules', 'quickcab');

  foreach ($geofence_rule_labels as $key => $value) {
    $geofence_rule_labels[$key] = sprintf($value, $singular_name, $plural_name);
  }

  $geofence_rule_post_type_args = array(
    'labels'             => $geofence_rule_labels,
    'public'             => false,
    'publicly_queryable' => true,
    'show_ui'            => true,
    'show_in_menu'       => 'edit.php?post_type=quickcab_vehicle',
    'query_var'          => true,
    'menu_icon'          => 'dashicons-dashboard',
    'map_meta_cap'       => true,
    'has_archive'        => true,
    'hierarchical'       => false,
    'supports'           => apply_filters('quickcab_geofence_rule_supports', array('title'))
  );

  register_post_type(
    'geofence_rule',
    apply_filters('quickcab_geofence_rule_post_type_args', $geofence_rule_post_type_args)
  );
}

function quickcab_register_fixed_addresses_post_type() {
  $fixed_address_labels = array(
    'name'                  => '%2$s',
    'singular_name'         => '%1$s',
    'add_new'               => esc_html__('Add New %1$s', 'quickcab'),
    'add_new_item'          => esc_html__('Add New %1$s', 'quickcab'),
    'edit_item'             => esc_html__('Edit %1$s', 'quickcab'),
    'new_item'              => esc_html__('New %1$s', 'quickcab'),
    'all_items'             => esc_html__('%2$s', 'quickcab'),
    'view_item'             => esc_html__('View %1$s', 'quickcab'),
    'search_items'          => esc_html__('Search %2$s', 'quickcab'),
    'not_found'             => esc_html__('No %2$s found', 'quickcab'),
    'not_found_in_trash'    => esc_html__('No %2$s found in Trash', 'quickcab'),
    'parent_item_colon'     => '',
    'menu_name'             => _x('%2$s', 'fixed addresses post type menu name', 'quickcab'),
    'featured_image'        => esc_html__('%1$s Image', 'quickcab'),
    'set_featured_image'    => esc_html__('Set %1$s Image', 'quickcab'),
    'remove_featured_image' => esc_html__('Remove %1$s Image', 'quickcab'),
    'use_featured_image'    => esc_html__('Use as %1$s Image', 'quickcab'),
    'filter_items_list'     => esc_html__('Filter %2$s list', 'quickcab'),
    'items_list_navigation' => esc_html__('%2$s list navigation', 'quickcab'),
    'items_list'            => esc_html__('%2$s list', 'quickcab'),
  );

  $singular_name = esc_html__('Fixed Address', 'quickcab');
  $plural_name = esc_html__('Fixed Addresses', 'quickcab');

  foreach ($fixed_address_labels as $key => $value) {
    $fixed_address_labels[$key] = sprintf($value, $singular_name, $plural_name);
  }

  $fixed_address_post_type_args = array(
    'labels'             => apply_filters('quickcab_fixed_address_post_type_labels', $fixed_address_labels),
    'public'             => false,
    'publicly_queryable' => true,
    'show_ui'            => true,
    'show_in_menu'       => 'edit.php?post_type=quickcab_vehicle',
    'query_var'          => true,
    'menu_icon'          => 'dashicons-dashboard',
    'map_meta_cap'       => true,
    'has_archive'        => true,
    'hierarchical'       => false,
    'supports'           => apply_filters('quickcab_fixed_address_supports', array('title'))
  );

  register_post_type(
    'fixed_address',
    apply_filters('quickcab_fixed_address_post_type_args', $fixed_address_post_type_args)
  );

  remove_post_type_support('fixed_address', 'title');
}

function quickcab_register_surge_price_post_type() {
  $surge_price_labels = array(
    'name'                  => esc_html__('%2$s', 'quickcab'),
    'singular_name'         => esc_html__('%1$s', 'quickcab'),
    'add_new'               => esc_html__('Add New', 'quickcab'),
    'add_new_item'          => esc_html__('Add New %1$s', 'quickcab'),
    'edit_item'             => esc_html__('Edit %1$s', 'quickcab'),
    'new_item'              => esc_html__('New %1$s', 'quickcab'),
    'all_items'             => esc_html__('%2$s', 'quickcab'),
    'view_item'             => esc_html__('View %1$s', 'quickcab'),
    'search_items'          => esc_html__('Search %2$s', 'quickcab'),
    'not_found'             => esc_html__('No %2$s found', 'quickcab'),
    'not_found_in_trash'    => esc_html__('No %2$s found in Trash', 'quickcab'),
    'parent_item_colon'     => '',
    'menu_name'             => _x('%2$s', 'surge price post type menu name', 'quickcab'),
    'featured_image'        => esc_html__('%1$s Image', 'quickcab'),
    'set_featured_image'    => esc_html__('Set %1$s Image', 'quickcab'),
    'remove_featured_image' => esc_html__('Remove %1$s Image', 'quickcab'),
    'use_featured_image'    => esc_html__('Use as %1$s Image', 'quickcab'),
    'filter_items_list'     => esc_html__('Filter %2$s list', 'quickcab'),
    'items_list_navigation' => esc_html__('%2$s list navigation', 'quickcab'),
    'items_list'            => esc_html__('%2$s list', 'quickcab'),
  );

  $singular_name = esc_html__('Surce Pricing Rule', 'quickcab');
  $plural_name = esc_html__('Surge Pricing Rules', 'quickcab');

  foreach ($surge_price_labels as $key => $value) {
    $surge_price_labels[$key] = sprintf($value, $singular_name, $plural_name);
  }

  $surge_price_post_type_args = array(
    'labels'             => $surge_price_labels,
    'public'             => false,
    'publicly_queryable' => true,
    'show_ui'            => true,
    'show_in_menu'       => 'edit.php?post_type=quickcab_vehicle',
    'query_var'          => true,
    'menu_icon'          => 'dashicons-dashboard',
    'map_meta_cap'       => true,
    'has_archive'        => true,
    'hierarchical'       => false,
    'supports'           => apply_filters('quickcab_surge_price_supports', array('title'))
  );

  register_post_type(
    'surge_price_rule',
    apply_filters('quickcab_surge_price_rule_post_type_args', $surge_price_post_type_args)
  );
}

function quickcab_register_checkout_field_post_type() {
  $checkout_field_labels = array(
    'name'                  => esc_html__('%2$s', 'quickcab'),
    'singular_name'         => esc_html__('%1$s', 'quickcab'),
    'add_new'               => esc_html__('Add New', 'quickcab'),
    'add_new_item'          => esc_html__('Add New %1$s', 'quickcab'),
    'edit_item'             => esc_html__('Edit %1$s', 'quickcab'),
    'new_item'              => esc_html__('New %1$s', 'quickcab'),
    'all_items'             => esc_html__('%2$s', 'quickcab'),
    'view_item'             => esc_html__('View %1$s', 'quickcab'),
    'search_items'          => esc_html__('Search %2$s', 'quickcab'),
    'not_found'             => esc_html__('No %2$s found', 'quickcab'),
    'not_found_in_trash'    => esc_html__('No %2$s found in Trash', 'quickcab'),
    'parent_item_colon'     => '',
    'menu_name'             => _x('%2$s', 'custom checkout field post type menu name', 'quickcab'),
    'featured_image'        => esc_html__('%1$s Image', 'quickcab'),
    'set_featured_image'    => esc_html__('Set %1$s Image', 'quickcab'),
    'remove_featured_image' => esc_html__('Remove %1$s Image', 'quickcab'),
    'use_featured_image'    => esc_html__('Use as %1$s Image', 'quickcab'),
    'filter_items_list'     => esc_html__('Filter %2$s list', 'quickcab'),
    'items_list_navigation' => esc_html__('%2$s list navigation', 'quickcab'),
    'items_list'            => esc_html__('%2$s list', 'quickcab'),
  );

  $singular_name = esc_html__('Custom Checkout Field', 'quickcab');
  $plural_name = esc_html__('Custom Checkout Fields', 'quickcab');

  foreach ($checkout_field_labels as $key => $value) {
    $checkout_field_labels[$key] = sprintf($value, $singular_name, $plural_name);
  }

  $checkout_field_post_type_args = array(
    'labels'             => $checkout_field_labels,
    'public'             => false,
    'publicly_queryable' => true,
    'show_ui'            => true,
    'show_in_menu'       => 'edit.php?post_type=quickcab_vehicle',
    'query_var'          => true,
    'menu_icon'          => 'dashicons-dashboard',
    'map_meta_cap'       => true,
    'has_archive'        => true,
    'hierarchical'       => false,
    'supports'           => apply_filters('quickcab_checkout_field_supports', array('title'))
  );

  register_post_type(
    'qc_checkout_field',
    apply_filters('quickcab_checkout_field_post_type_args', $checkout_field_post_type_args)
  );

  remove_post_type_support('qc_checkout_field', 'title');
}

function quickcab_register_booking_form_post_type() {
  $booking_form_labels = array(
    'name'                  => esc_html__('%2$s', 'quickcab'),
    'singular_name'         => esc_html__('%1$s', 'quickcab'),
    'add_new'               => esc_html__('Add New', 'quickcab'),
    'add_new_item'          => esc_html__('Add New %1$s', 'quickcab'),
    'edit_item'             => esc_html__('Edit %1$s', 'quickcab'),
    'new_item'              => esc_html__('New %1$s', 'quickcab'),
    'all_items'             => esc_html__('%2$s', 'quickcab'),
    'view_item'             => esc_html__('View %1$s', 'quickcab'),
    'search_items'          => esc_html__('Search %2$s', 'quickcab'),
    'not_found'             => esc_html__('No %2$s found', 'quickcab'),
    'not_found_in_trash'    => esc_html__('No %2$s found in Trash', 'quickcab'),
    'parent_item_colon'     => '',
    'menu_name'             => _x('%2$s', 'custom checkout field post type menu name', 'quickcab'),
    'featured_image'        => esc_html__('%1$s Image', 'quickcab'),
    'set_featured_image'    => esc_html__('Set %1$s Image', 'quickcab'),
    'remove_featured_image' => esc_html__('Remove %1$s Image', 'quickcab'),
    'use_featured_image'    => esc_html__('Use as %1$s Image', 'quickcab'),
    'filter_items_list'     => esc_html__('Filter %2$s list', 'quickcab'),
    'items_list_navigation' => esc_html__('%2$s list navigation', 'quickcab'),
    'items_list'            => esc_html__('%2$s list', 'quickcab'),
  );

  $singular_name = esc_html__('Booking Form', 'quickcab');
  $plural_name = esc_html__('Booking Forms', 'quickcab');

  foreach ($booking_form_labels as $key => $value) {
    $booking_form_labels[$key] = sprintf($value, $singular_name, $plural_name);
  }

  $booking_form_post_type_args = array(
    'labels'             => $booking_form_labels,
    'public'             => false,
    'publicly_queryable' => true,
    'show_ui'            => true,
    'show_in_menu'       => 'edit.php?post_type=quickcab_vehicle',
    'query_var'          => true,
    'menu_icon'          => 'dashicons-dashboard',
    'map_meta_cap'       => true,
    'has_archive'        => true,
    'hierarchical'       => false,
    'supports'           => apply_filters('quickcab_booking_form_supports', array('title'))
  );

  register_post_type(
    'qc_booking_form',
    apply_filters('quickcab_booking_form_post_type_args', $booking_form_post_type_args)
  );
}
